﻿##### Scriptinfo ##################################################################################
# Scriptreihe     PrivilegedADUser-Analyse
# Datum:          2019-03-01
# Version:        V1.16
# Programmierer:  Stephan Walther
###################################################################################################

# Elevation
    try   { $ScriptDir = [System.IO.Path]::GetDirectoryName($myInvocation.MyCommand.Definition) }
    catch { $ScriptDir = Split-Path -Path $psise.CurrentFile.FullPath -Parent }
          
    $CurrentUser = [Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()
    If (-not ($CurrentUser).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator")) {
        if ($host.name -eq 'Windows PowerShell ISE Host') {
            Write-Host "Bitte ISE als Administrator starten!"
            break
        } else {
            $AdminPS = new-object System.Diagnostics.ProcessStartInfo "PowerShell"
            $AdminPS.Arguments = $myInvocation.MyCommand.Definition
            $AdminPS.Verb = "runas"
            $adminps.WorkingDirectory = $ScriptDir
        
            [System.Diagnostics.Process]::Start($AdminPS)    
            exit 
        }
    }
    
# Variablen
    $globaleVariablen = @('AdminGroups','Benutzer','SchedTasks','Services','MemberServer','HTMLMail','ScriptStart','PP','AdminGruppenOUs','AdminGruppenFLT','AdminAusnahmen')
        
    if (Test-Path -Path "$ScriptDir\PrivilegedADUser-Analyse.ps1") {        
        Set-Location -Path $ScriptDir    
        $ScriptVersion = ((Get-Content -Path "$ScriptDir\PrivilegedADUser-Analyse.ps1" | Select-String '# Version:' | Where-Object { $_ -notlike '*get-content*' }) -split ':')[1].trim()
    }
    
# Funktionen
    function lese-Variablen {
        param($Datei,$VarPrefix = "WS_") 

        $VarPrefixInt = $VarPrefix

        Get-Content -Path $Datei | ForEach-Object {
            $x = @{}
                       
            if ($_ -notlike "'*" -and $_.length -gt 0) {                         # keine Kommentare oder Leerzeilen verarbeiten
                if ($_ -notlike '`[*') {                                         # Block oder Variable?
                    if ($_ -like '*+=*') {                                       # SingleVar oder Array?
                        $x.name,$x.wert = $_  -split '\+='

                        $VarName =  $x.name.trim()
                        $VarWert = ($x.wert.trim() -split '#')[0].trim()

                        if ($VarWert.length -gt 0) {
                            if (-not (Get-Variable -Scope global | Where-Object {$_.name -eq "$VarPrefix$VarName"})) {
                                New-Variable -Name "$VarPrefixInt$VarName" -Value @() -Scope global
                            }
                
                            Set-Variable -Name "$VarPrefixInt$VarName" -Scope global -Value ((Get-Variable -Name "$VarPrefixInt$VarName").Value += $VarWert)
                        }
                    } else {
                        $x.name,$x.wert = $_ -split '='

                        $VarName = $x.name.trim()
                        $VarWert = ($x.wert.trim() -split '#')[0].trim()

                        if ($VarWert -like '*$(*)*') { $VarWert = Invoke-Expression -Command $VarWert }

                        Get-Variable -Scope global | Where-Object {$_.name -eq "$VarPrefixInt$VarName"} | Remove-Variable -Scope global
                        New-Variable -Name "$VarPrefixInt$VarName" -Value $VarWert -Scope global
                    }
                } else {
                    if ($_ -eq '[ComputerGruppen]') { $VarPrefixInt = "CG_" } else { $VarPrefixInt = $VarPrefix }
                }
            }
        }
    }

    function ermittle-GruppenMitglied {
        param($Gruppe)

        Get-ADGroupMember -Identity $Gruppe | ForEach-Object {
            if ($_.objectclass -eq 'user') {
                $_.samaccountname 
            } 
            if ($_.objectclass -eq 'group') {        
                ermittle-GruppenMitglied -Gruppe $_.samaccountname 
            }
        }
    }

    function finde-MemberServer {
        Write-Host "finde MemberServer" -ForegroundColor White
        $Server = Get-ADComputer -Filter "operatingsystem -like '*server*'" -Properties operatingsystem,whenchanged,PasswordLastSet
        $Server = $Server | Where-Object {$_.PasswordLastSet -and $_.whenchanged -gt (Get-Date).AddDays(-60) -and $_.PasswordLastSet -gt (Get-Date).AddDays(-60)}
        Write-Host "   $($Server.Count) Hosts gefunden" -ForegroundColor green

        $Server = $Server | Select-Object -Property DNSHostname,Enabled,OperatingSystem,PasswordLastSet
        $Server | Add-Member -MemberType NoteProperty -Name AdminTasks -Value 0
        $Server | Add-Member -MemberType NoteProperty -Name AdminServices -Value 0
        $Server | Add-Member -MemberType NoteProperty -Name PSRemoting -Value ""

        $global:MemberServer = $Server
    }

    function finde-AdminGruppen {
        Write-Host "finde AdminGruppen" -ForegroundColor White        

        $global:AdminGroups += Get-ADGroup -Filter 'Admincount -eq 1' | Select-Object -Property *,@{n='Level';e={1}}
    
        $AdminGruppenOUs | ForEach-Object {
            $weitereGruppen = Get-ADGroup -Filter ($AdminGruppenFLT -join ' -or ') -SearchBase $_ | Select-Object -Property *,@{n='Level';e={2}}
            $global:AdminGroups += $weitereGruppen | Where-Object {$AdminGroups.SamAccountname -notcontains $_.Samaccountname}
        }

        $global:AdminGroups | Add-Member -MemberType NoteProperty -Name Mitglieder -Value @()
        $global:AdminGroups | ForEach-Object {
            $_.Mitglieder = (ermittle-GruppenMitglied -Gruppe $_.SamAccountName | Sort-Object -Unique) -join ', '
        }

        Write-Host "   $($AdminGroups.Count) Gruppen gefunden" -ForegroundColor green
    }

    function finde-Administratoren {
        Write-Host "finde Administratoren" -ForegroundColor White

        $AdminGroups | Sort-Object -Property Level | ForEach-Object {
            $Level = $_.Level
            ermittle-GruppenMitglied -Gruppe $_.samaccountname | Select-Object -Unique | ForEach-Object {
                if ($Benutzer.Benutzer -notcontains $_) {
                    $out = New-Object -TypeName psobject
                    $out | Add-Member -MemberType NoteProperty -Name Benutzer -Value $_
                    $out | Add-Member -MemberType NoteProperty -Name Level    -Value $Level
                    $global:Benutzer += $out
                }
            }
        } 

        $out = New-Object -TypeName psobject
        $out | Add-Member -MemberType NoteProperty -Name Benutzer -Value krbtgt
        $out | Add-Member -MemberType NoteProperty -Name Level    -Value 1
        $global:Benutzer += $out

        $global:Benutzer | Add-Member -MemberType NoteProperty -Name CannotChangePassword -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name PasswordNeverExpires -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name PasswordNotRequired -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name PasswordLastSet -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name PasswordExpireDate -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name IsProtectedUser -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name enabled -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name description -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name AccountNotDelegated -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name SCLogonRequired -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name LastLogonDate -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name Policy -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name Lockout -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name PWcomplex -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name PWreversible -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name PWlength -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name PWhistory -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name PWminAge -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name KRBpreAuth -Value ''
                
        $global:Benutzer | ForEach-Object {
            $ADUser = Get-ADUser -Identity $_.Benutzer -Properties AccountNotDelegated,SmartcardLogonRequired,CannotChangePassword,PasswordNeverExpires,PasswordNotRequired,PasswordLastSet,enabled,description,msDS-UserPasswordExpiryTimeComputed,LastLogonDate,AllowReversiblePasswordEncryption,DoesNotRequirePreAuth

            if (Get-ADAccountAuthorizationGroup -Identity $ADUser | Where-Object {($_.SID -split '-')[-1] -eq 525}) {
                $IStProtectedUser = $true
            } else {
                $IStProtectedUser = $false
            }

            try {
                $ExpireDate = (get-date ([datetime]::FromFileTime($ADUser."msDS-UserPasswordExpiryTimeComputed")) -Format u -ErrorAction SilentlyContinue) -replace 'Z'
            }
            catch {
                $ExpireDate = $null
            }

            $_.CannotChangePassword = $ADUser.CannotChangePassword
            $_.PasswordNeverExpires = $ADUser.PasswordNeverExpires
            $_.PasswordNotRequired  = $ADUser.PasswordNotRequired
            $_.PasswordLastSet      = (get-date -date $ADUser.PasswordLastSet -Format u -ErrorAction SilentlyContinue) -replace 'Z'
            $_.PasswordExpireDate   = $ExpireDate
            $_.IsProtectedUser      = $IStProtectedUser
            $_.enabled              = $ADUser.enabled
            $_.description          = $ADUser.description
            $_.AccountNotDelegated  = $ADUser.AccountNotDelegated
            $_.SCLogonRequired      = $ADUser.SmartcardLogonRequired
            $_.PWReversible         = $ADUser.AllowReversiblePasswordEncryption
            $_.KRBpreAuth           = -not $ADUser.DoesNotRequirePreAuth
            $_.LastLogonDate        = if ($ADUser.LastLogonDate) {(get-date -date $ADUser.LastLogonDate -Format u) -replace 'z'} else {'never'}
        
            $PSO = Get-ADUserResultantPasswordPolicy -Identity $_.Benutzer
            if ($PSO) {
                $_.Policy             = $PSO.Name 
                $PED = (Get-Date -date $_.PasswordLastSet).AddDays($PSO.MaxPasswordAge.TotalDays)
                $_.PasswordExpireDate = (get-date -date $PED -Format u -ErrorAction SilentlyContinue) -replace 'Z'

                $_.Lockout            = $PSO.LockoutThreshold
                $_.PWcomplex          = $PSO.ComplexityEnabled
                if ($_.PWreversible -eq $false) { $_.PWreversible       = $PSO.ReversibleEncryptionEnabled }
                $_.PWlength           = $PSO.MinPasswordLength
                $_.PWhistory          = $PSO.PasswordHistoryCount
                $_.PWminAge           = $PSO.MinPasswordAge.TotalDays
            } else {
                $_.Policy       = "GPO"
                $_.Lockout      = $PP.ADlockoutThreshold
                $_.PWcomplex    = $PP.PWcomplex
                if ($_.PWreversible -eq $false) { $_.PWreversible = $PP.PWreversible }
                $_.PWlength     = $PP.ADminPwdLength
                $_.PWhistory    = $PP.ADpwdHistoryLength
                $_.PWminAge     = $PP.ADminPwdAge_Day
            }
        }
        
        Write-Host "   $($Benutzer.Count) Admins gefunden" -ForegroundColor green
    }

    function ermittle-ADUsername {
        param(
            [parameter(Mandatory=$true)] [string] $Eingabe
        )
    
        try {
            [System.Security.Principal.SecurityIdentifier] $Eingabe | Out-Null
        
            Get-ADObject -Filter "ObjectSID -eq '$Eingabe'" |
                Get-ADUser -ErrorAction Stop |
                    Select-Object -ExpandProperty SamAccountName
        }
        catch {
            $Eingabe
        }    
    }
       
    function finde-ScheduledTasks {
        $Function_Get_AllTaskSubFolders = @'
function Get-AllTaskSubFolders {
    [cmdletbinding()]
    param (
        # Set to use $Schedule as default parameter so it automatically list all files
        # For current schedule object if it exists.
        $FolderRef = $Schedule.getfolder("\")
    )
    if ($FolderRef.Path -eq "\") {
        $FolderRef
    }
    if (-not $RootFolder) {
        $ArrFolders = @()
        if(($Folders = $folderRef.getfolders(1))) {
            $Folders | ForEach-Object {
                $ArrFolders += $_
                if($_.getfolders(1)) {
                    Get-AllTaskSubFolders -FolderRef $_
                }
            }
        }
        $ArrFolders
    }
}
'@

        $Function_Get_TaskTrigger = @'
function Get-TaskTrigger {
    [cmdletbinding()]
    param (
        $Task
    )
    $Out = @()
    if ($Triggers) {
        $Triggers | Get-Member -MemberType Property | ForEach-Object {
            $out += $Triggers.($_.Name)
        }
    }
    Return ($out -join ';')
}
'@

        $Function_Lese_ScheduledTask = @'
function Lese-ScheduledTask {        
    $ComputerName = $env:COMPUTERNAME
    $Schedule = New-Object -ComObject 'Schedule.Service'
        
    try {
        $Schedule.connect($ComputerName) 
        $AllFolders = Get-AllTaskSubFolders
        
        foreach ($Folder in $AllFolders) {
            $Tasks = $Folder.GetTasks(1)

            Foreach ($Task in $Tasks) {
                $Action = @()
                ([xml] $Task.xml).task.actions.exec | ForEach-Object {
                    $Action += $_.command + ' ' + $_.Arguments
                }
                $Action = $Action -join ';'
                
                New-Object -TypeName PSCustomObject -Property @{
	                    'Name' = $Task.name
                        'Path' = $Task.path
                        'State' = switch ($Task.State) {
                            0 {'Unknown'}
                            1 {'Disabled'}
                            2 {'Queued'}
                            3 {'Ready'}
                            4 {'Running'}
                            Default {'Unknown'}
                        }
                        'Enabled' = $Task.enabled
                        'LastRunTime' = $Task.lastruntime
                        'LastTaskResult' = $Task.lasttaskresult
                        'NumberOfMissedRuns' = $Task.numberofmissedruns
                        'NextRunTime' = $Task.nextruntime
                        'Author' =  ([xml]$Task.xml).Task.RegistrationInfo.Author
                        'UserId' = ([xml]$Task.xml).Task.Principals.Principal.UserID
                        'Description' = ([xml]$Task.xml).Task.RegistrationInfo.Description
                        'Trigger' = Get-TaskTrigger -Task $Task
                        'LogonType' = ([xml]$Task.xml).Task.Principals.Principal.LogonType
                        'ComputerName' = $Schedule.TargetServer
                        'Action' = $Action
                    }
            }                
        }
        Write-Host "      $ComputerName" -ForegroundColor green 
    } 
    catch {
        $Fehler = $error[0].exception.message
        Write-Host "      $ComputerName - $Fehler" -ForegroundColor Magenta 
            
    }
}
'@

        Write-Host "finde ScheduledTasks" -ForegroundColor White
               
        Write-Host "   sammle Task-Informationen über WinRM" -ForegroundColor White
        $Ergebnis = @()
        $error.Clear()
        
        $AnzahlServer = $MemberServer.count

        $Ergebnis += Invoke-Command -ComputerName ($MemberServer).DNSHostName -ScriptBlock {
            param($Function_Get_AllTaskSubFolders,$Function_Get_TaskTrigger,$Function_Lese_ScheduledTask)

            Invoke-Expression $Function_Get_AllTaskSubFolders
            Invoke-Expression $Function_Get_TaskTrigger
            Invoke-Expression $Function_Lese_ScheduledTask

            Lese-ScheduledTask
    
        } -ArgumentList $Function_Get_AllTaskSubFolders,$Function_Get_TaskTrigger,$Function_Lese_ScheduledTask -ErrorAction SilentlyContinue
                
        $AnzahlErgebnisse = ($Ergebnis | Group-Object -Property ComputerName | measure-object).count
        
        if ($AnzahlErgebnisse -lt $AnzahlServer) {
            Write-Host "   $AnzahlErgebnisse/$AnzahlServer Server ausgelesen" -ForegroundColor Yellow
        } else {
            Write-Host "   $AnzahlErgebnisse/$AnzahlServer Server ausgelesen" -ForegroundColor Green
        }        
        Write-Host "   filtere Task-Informationen" -ForegroundColor White
        $Ergebnis | ForEach-Object {
            if (($_.userid -split '\\').count -gt 1) {
                $_.userid = ($_.userid -split '\\')[1]
            } 
            if ($_.userid.length -gt 0) {
                $_.userid = ermittle-ADUsername -Eingabe $_.Userid
            }
        }

        $global:SchedTasks = $Ergebnis | 
            Where-Object { $_.name -notlike '*Optimize Start Menu Cache*' `
                      -and $_.state -ne 'Disabled' `
                      -and $_.UserID -ne $null `
                      -and $_.LogonType -eq 'password' `
                      -and $_.UserID -notlike 'S-1-5-*' `
                      -and $_.name -notlike '*_Active Directory Diagnostics' `
                      -and $_.name -notlike '*User_Feed_Synchronization*' `
                      -and $_.name -notlike '*Adobe Flash Player*' `
                      -and $_.name -notlike '*{*' `
                      -and 'NT AUTHORITY\SYSTEM','SYSTEM' -notcontains $_.UserID} 
    }

    function finde-AdminsMitScheduledTasks {
        Write-Host "finde Admins mit ScheduledTasks" -ForegroundColor White
        
        $ServiceBenutzer = $SchedTasks | Group-Object -Property userid | Select-Object -Property name,count    
        
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name SchedTasks -Value $null

        $global:Benutzer | ForEach-Object {
            $SAM = $_.Benutzer        
            $_.SchedTasks = $ServiceBenutzer | Where-Object {$_.name -eq $SAM} | Select-Object -ExpandProperty count               
        }

        $Anzahl = ($Benutzer | Where-Object {$_.schedTasks -gt 0} | Measure-Object).count
        if ($Anzahl -eq 0) {
            Write-Host "   $Anzahl Benutzer mit ScheduledTask gefunden" -ForegroundColor green
        } else {
            Write-Host "   $Anzahl Benutzer mit ScheduledTask gefunden" -ForegroundColor Yellow
        }
    }

    function finde-Services {
        Write-Host "finde Services" -ForegroundColor White
        Write-Host "   sammle Service-Informationen über WinRM" -ForegroundColor White
        $Ergebnis = @()
        $error.Clear()
        
        $AnzahlServer = $MemberServer.count
        $Ergebnis += Invoke-Command -ComputerName ($MemberServer | Select-Object -ExpandProperty DNSHostName) -ErrorAction SilentlyContinue -ScriptBlock {
            Write-Host "      $($env:COMPUTERNAME)" -ForegroundColor green

            Get-WmiObject -Class win32_service |         
                Select-Object -Property @{n='Server'    ;e={$_.Systemname}},
                                        @{n='Service'   ;e={$_.DisplayName}},
                                        @{n='Pfad'      ;e={$_.PathName}},
                                        @{n='State'     ;e={$_.State}},
                                        @{n='StartMode' ;e={$_.StartMode}},
                                        @{n='Benutzer'  ;e={$_.StartName}}
        }
        
        $AnzahlErgebnisse = ($Ergebnis | Group-Object -Property Server | Measure-Object).count 
        if ($AnzahlErgebnisse -lt $AnzahlServer) {
            Write-Host "   $AnzahlErgebnisse/$AnzahlServer Server ausgelesen" -ForegroundColor Yellow
        } else {
            Write-Host "   $AnzahlErgebnisse/$AnzahlServer Server ausgelesen" -ForegroundColor Green
        }             
        
        $Ergebnis | ForEach-Object {
            if ($_.Benutzer.length -gt 0) {
                $_.Benutzer = ermittle-ADUsername -Eingabe $_.Benutzer
            }
            if (($_.Benutzer -split '\\').count -gt 1) {
                $_.Benutzer = ($_.Benutzer -split '\\')[-1]
            }
            if ($_.Benutzer -like '*@*') {
                $_.Benutzer = ($_.Benutzer -split '@')[0]
            }
        }

        Write-Host "   filtere Service-Informationen" -ForegroundColor White
        $global:Services = @()
        $global:Services = $Ergebnis | Where-Object {$_.Benutzer -notlike 'NT AUTHORITY*'}
        $global:Services = $Services | Where-Object {$_.Benutzer -notlike 'NT-AUTORITÄT*'}
        $global:Services = $Services | Where-Object {$_.Benutzer -notlike 'NT SERVICE*'}
        $global:Services = $Services | Where-Object {$_.Benutzer -notlike '.\*'}
    
        $global:Services | ForEach-Object {            
            $_.Benutzer = $_.Benutzer -replace "$($env:USERDOMAIN)\\"
            $_.Benutzer = $_.Benutzer -replace '\.\\'
            $_.Benutzer = ($_.Benutzer -split '@')[0]
        }
    }

    function finde-AdminsMitServices {
        Write-Host "finde Admins, die Services ausführen" -ForegroundColor White
        
        $global:Services = $Services | Where-Object {$_.StartMode -ne 'disabled'}
        $global:Services = $Services | Where-Object {$_.Benutzer -ne 'localsystem'}

        $global:Benutzer | Add-Member -MemberType NoteProperty -Name ServiceUser -Value $null

        $Services | Group-Object -Property Benutzer | ForEach-Object {
            $SAM = $_.name  
            $Anz = $_.count
            $global:Benutzer | Where-Object {$_.Benutzer -eq $SAM} | ForEach-Object {$_.ServiceUser = $Anz}
        }

        $Anzahl = ($Benutzer | Where-Object {$_.ServiceUser -gt 0} | Measure-Object).count
        if ($Anzahl -eq 0) {
            Write-Host "   $Anzahl Benutzer mit Service gefunden" -ForegroundColor green
        } else {
            Write-Host "   $Anzahl Benutzer mit ScheduledTask gefunden" -ForegroundColor Yellow
        }
    }

    function erstelle-FarbFilter {
        param(
            [parameter(Mandatory=$true)]  [string] $Spalte,
            [parameter(Mandatory=$false)] [string] $rot   = '$false',
            [parameter(Mandatory=$false)] [string] $gelb  = '$false',
            [parameter(Mandatory=$false)] [string] $gruen = '$false',
            [parameter(Mandatory=$false)] [string] $blau  = '$false',
            [parameter(Mandatory=$false)] [switch] $neuerFilter = $false
        )

        if ((Get-Variable -Scope global | Where-Object { $_.Name -eq "FarbFilter" }) -and $neuerFilter) {
            Remove-Variable -Name "FarbFilter" -Scope global
        
        }

        if ((Get-Variable -Scope global | Where-Object { $_.Name -eq "FarbFilter" }) -eq $null) {
            New-Variable -Name "FarbFilter" -Value @() -Scope global
        }

        if ($FarbFilter | Where-Object { $_.Spalte -eq $Spalte }) {
            $Rest = $FarbFilter | Where-Object { $_.Spalte -ne $Spalte }           
            $global:FarbFilter = @()
            $global:FarbFilter += $Rest
        } 

        $Filter = New-Object -TypeName PSObject
        $Filter | Add-Member -MemberType NoteProperty -Name Spalte -Value $Spalte    
        $Filter | Add-Member -MemberType NoteProperty -Name rot    -Value $rot
        $Filter | Add-Member -MemberType NoteProperty -Name gelb   -Value $gelb
        $Filter | Add-Member -MemberType NoteProperty -Name gruen  -Value $gruen
        $Filter | Add-Member -MemberType NoteProperty -Name blau   -Value $blau
            
        $global:FarbFilter += $Filter    
    }    

    function erstelle-HTMLReport {
        param(
            [cmdletbinding()]        
            [parameter(Mandatory=$true)]  [Object] $Daten      = @(),
            [parameter(Mandatory=$false)] [string] $Headline   = '',
            [parameter(Mandatory=$false)] [switch] $mitStart   = $false,
            [parameter(Mandatory=$false)] [switch] $mitEnde    = $false,
        
            [parameter(Mandatory=$false,ParametersetName='Set1')] [validateset("rot","gruen","blau")] [String] $Muster = '',
            [parameter(Mandatory=$false,ParametersetName='Set1')] [object]   $Farbdefinition = @(),
            [parameter(Mandatory=$false,ParametersetName='Set2')] [string]   $gruen          = '',
            [parameter(Mandatory=$false,ParametersetName='Set2')] [string]   $gelb           = '',
            [parameter(Mandatory=$false,ParametersetName='Set2')] [string]   $rot            = '',
            [parameter(Mandatory=$false,ParametersetName='Set2')] [string]   $blau           = '',
            

            [parameter(Mandatory=$false)] [string[]] $Spalten    = '',
            [parameter(Mandatory=$false)] [string]   $Dateiname  = ''
        )

        $Ausgabe = @()

        # erzeuge HTML-Header
            if ($mitStart) {
                $Ausgabe += "<HTML>"
                $Ausgabe += "<HEAD>"
                $Ausgabe += "<STYLE>"
                $Ausgabe += "   BODY{font-family: Arial; font-size: 8pt;}"
                $Ausgabe += "   H1{font-size: 16px;}"
                $Ausgabe += "   H2{font-size: 14px;}"
                $Ausgabe += "   H3{font-size: 12px;}"
                $Ausgabe += "   TABLE{border: 1px solid black; border-collapse: collapse; font-size: 8pt;}"
                $Ausgabe += "   TH{border: 1px solid black; background: #dddddd; padding: 5px; color: #000000;}"
                $Ausgabe += "   TD{border: 1px solid black; padding: 5px; }"
                $Ausgabe += "   td.pass{background: #40FF00;}"
                $Ausgabe += "   td.warn{background: #FFE600;}"
                $Ausgabe += "   td.fail{background: #FF0000; color: #ffffff;} "
                $Ausgabe += "   td.info{background: #85D4FF;}"                            
                $Ausgabe += "   td.musterR1{background: #F6CECE;}"
                $Ausgabe += "   td.musterR2{background: #FA5858;}"
                $Ausgabe += "   td.musterG1{background: #D8F6CE;}"
                $Ausgabe += "   td.musterG2{background: #ACFA58;}"
                $Ausgabe += "   td.musterB1{background: #FFFFFF;}"
                $Ausgabe += "   td.musterB2{background: #A9F5F2;}"
                $Ausgabe += "</STYLE>"
                $Ausgabe += "</HEAD>"
                $Ausgabe += "<BODY>"                
            } 

        # erzeuge Headline
            if ($Headline -ne '') {$Ausgabe += "<h3>$Headline</h3>"}

        # generiere Kopfzeile der Ausgabetabelle
            $Spaltennamen = @()
            if (($Daten).length -ne 0) {
                if ($Spalten[0] -eq '') {        
                    $Daten | Get-Member -MemberType NoteProperty | ForEach-Object {
                        $Spalte = $_.Name 
                        $Spaltennamen += "<TH>$Spalte</TH>"        
                    }
                } else { 
                    $Spalten | ForEach-Object {
                        $Spalte = ($_ -split ':')[-1]
                        $Spaltennamen += "<TH>$Spalte</TH>"        
                    }
                } 
            }

        # erzeuge Zeilen der Ausgabetabelle        
            if (($Daten).length -ne 0) {
                $Ausgabe += "   <TABLE>"
                $Ausgabe += "      </TR>" 
                $Spaltennamen | ForEach-Object { $Ausgabe += "         $_"  }
                $Ausgabe += "      </TR>" 

                $Muster1 = "muster$($Muster[0])1"
                $Muster2 = "muster$($Muster[0])2"
                                        
                $Daten | ForEach-Object {
                    $DatenZeile = $_                 
                    $Ausgabe += '      <TR>' 
                    $stat = 'leer'
                    
                    if ($Muster -ne '') {
                        if ($FarbeAn) {
                            $stat    = $Muster1
                            $FarbeAn = $false
                        } else {
                            $stat    = $Muster2
                            $FarbeAn = $true
                        }   
                    } else {
                        if ($blau  -ne '') { Invoke-Expression ('if (' + $blau  + ' ) {$stat = "info"}') }
                        if ($gruen -ne '') { Invoke-Expression ('if (' + $gruen + ' ) {$stat = "pass"}') }
                        if ($gelb  -ne '') { Invoke-Expression ('if (' + $gelb  + ' ) {$stat = "warn"}') }
                        if ($rot   -ne '') { Invoke-Expression ('if (' + $rot   + ' ) {$stat = "fail"}') }
                    }

                    if ($Spalten[0] -eq '') {
                        $Daten | Get-Member -MemberType NoteProperty | ForEach-Object {
                            $Spalte = $_.Name
                            $Ausgabe += '         <TD class="' + $stat + '">' + $DatenZeile.$Spalte + '</TD>'
                        }
                    } else {
                        $Spalten | ForEach-Object {
                            $Spalte = ($_ -split ':')[0]   
                            $curstat = $stat

                            $Rule = $Farbdefinition | Where-Object { $_.Spalte -eq $Spalte }                         
                            if ($Rule) {
                                Invoke-Expression ('if (' + ($Rule.blau  -replace '\$_','$DatenZeile') + ' ) {$curstat = "info"}')    
                                Invoke-Expression ('if (' + ($Rule.gruen -replace '\$_','$DatenZeile') + ' ) {$curstat = "pass"}')    
                                Invoke-Expression ('if (' + ($Rule.gelb  -replace '\$_','$DatenZeile') + ' ) {$curstat = "warn"}')                                
                                Invoke-Expression ('if (' + ($Rule.rot   -replace '\$_','$DatenZeile') + ' ) {$curstat = "fail"}')    
                            } 

                            $Ausgabe += '         <TD class="' + $curstat + '">' + $DatenZeile.$Spalte + '</TD>'
                        }
                    }           
                    $Ausgabe += '      </TR>' 
                }
                $Ausgabe += '   </TABLE>'
            } 
        
        # schreibe Ende
            if ($mitEnde) {
                $Ausgabe += '</BODY>'
                $Ausgabe += '</HTML>'
            }
        
        # Ausgabe
            if ($Dateiname -eq '') {
                RETURN ($Ausgabe -join "`r`n")
            } else {        
                ($Ausgabe -join "`r`n") | Out-File -FilePath $Dateiname -Encoding default
            }    
    }

    function erstelle-HTML {
        Write-Host "erstelle HTML-Bericht" -ForegroundColor White
        Set-Location $ScriptDir
        $HTML = @()
        
        # Variablen
            $Admins          = ($Benutzer | Where-Object {$_.Level -le 2}).Benutzer
            $AnzahlAdmins    = ($Benutzer | Where-Object {$_.Level -le 2}).count            
        
        # AdminGruppen                            
            $Tab = $AdminGroups | Where-Object {$_.Level -lt 3} | 
                    Sort-Object -Property Level,SamAccountName | 
                        Select-Object -Property Samaccountname,Level,Mitglieder,@{n='RID';e={($_.SID -split '-')[-1]}}

            if ($Tab) {
                erstelle-FarbFilter -neuerFilter -Spalte Level -gelb '$_.Level -eq 1 -and $_.Mitglieder.length -gt 0' -gruen '$_.Level -eq 2 -and $_.Mitglieder.length -gt 0' -rot '518,519 -contains $_.RID -and $_.Mitglieder.length -gt 0'
                erstelle-FarbFilter -Spalte Mitglieder         -gelb '$_.Level -eq 1 -and $_.Mitglieder.length -gt 0' -gruen '$_.Level -eq 2 -and $_.Mitglieder.length -gt 0' -rot '518,519 -contains $_.RID -and $_.Mitglieder.length -gt 0'
                
                $HTML += erstelle-HTMLReport `
                    -Daten $Tab `
                    -Headline 'AdminGruppen Level 1 und 2' `
                    -Spalten 'samaccountname:Gruppe','Level','Mitglieder' `
                    -Farbdefinition $FarbFilter `
                    -Muster blau `
                    -mitStart                 
            }
        
        # FarbFilter
            erstelle-FarbFilter -Spalte Level                -gelb  '$_.Level -eq 1' -neuerFilter
            erstelle-FarbFilter -Spalte CannotChangePassword -rot   '$_.CannotChangePassword -eq $true'
            erstelle-FarbFilter -Spalte PasswordNeverExpires -rot   '$_.PasswordNeverExpires -eq $true -and $_.Benutzer -ne "KRBTGT"'
            erstelle-FarbFilter -Spalte Lockout              -rot   '$_.Lockout -eq 0'
            erstelle-FarbFilter -Spalte PasswordExpireDate   -rot   '($_.PWExpireInDays -eq "" -or $_.PWExpireInDays -le $PWDAblauf_Achtung) -and $_.SCLogonRequired -eq $false' `
                                                             -gelb  '$_.PWExpireInDays -le $PWDAblauf_Warnung -and $_.SCLogonRequired -eq $false'
            erstelle-FarbFilter -Spalte IsProtectedUser      -rot   '$_.Benutzer -ne "KRBTGT" -and $_.Level -eq 1 -and $_.IsProtectedUser -eq $false' `
                                                             -gruen '$_.IsProtectedUser -eq $true'
            erstelle-FarbFilter -Spalte SchedTasks           -rot   '$_.SchedTasks  -gt 0'
            erstelle-FarbFilter -Spalte ServiceUser          -rot   '$_.ServiceUser -gt 0'
            erstelle-FarbFilter -Spalte PWcomplex            -rot   '$_.PWcomplex -eq $false -and $_.PWlength -le $PWsichereLaenge' `
                                                             -gruen '$_.PWcomplex -eq $false -and $_.PWlength -gt $PWsichereLaenge'
            erstelle-FarbFilter -Spalte PWreversible         -rot   '$_.PWreversible -eq $true'
            erstelle-FarbFilter -Spalte PWlength             -rot   '$_.PWlength -lt $PWmindestLaenge -and $_.SCLogonRequired -eq $false' 
            erstelle-FarbFilter -Spalte PWhistory            -rot   '$_.PWhistory -eq 0 -or $_.PWminAge -eq 0' `
                                                             -gelb  '$_.PWhistory -le $PWhistory_Warnung'
            erstelle-FarbFilter -Spalte KRBpreAuth           -rot   '$_.PWreversible -eq $true'
            erstelle-FarbFilter -Spalte SCLogonRequired      -gruen '$_.SCLogonRequired -eq $true'
            erstelle-FarbFilter -Spalte LastLogonDate        -rot   '$_.LastLogonDate -eq "never"'

        # gefährdete Admins (nicht ausgenommen)
            $Tab = $Benutzer | 
                Where-Object {$_.Level -le 2 -and $_.BewertungWert -ne 'gruen' -and $AdminAusnahmen -notcontains $_.Benutzer} | 
                    Sort-Object -Property Level,Benutzer
            $AnzahlAdminsBad = ($Tab | measure-object).count
            
            if ($Tab) {
                $HTML += erstelle-HTMLReport `
                    -Daten $Tab `
                    -Headline "$AnzahlAdminsBad/$AnzahlAdmins privilegierte Accounts sind gefaehrdet" `
                    -Spalten ('Benutzer','Level','Policy',,'Lockout','IsProtectedUser:ProtectedUser','Enabled','Description','SchedTasks','ServiceUser','LastLogonDate:LastLogon',
                              'CannotChangePassword:PWfixed','PasswordNeverExpires:PWendless','PasswordLastSet:PWlastset','PasswordExpireDate:PWexpire',
                              'PWcomplex','PWreversible','PWlength','PWhistory','PWminAge','KRBpreAuth','SCLogonRequired:requireSC','Bewertung') `
                    -Farbdefinition $FarbFilter `
                    -Muster rot
            }

        # gefährdete Admins (ausgenommen)
            $Tab = $Benutzer | 
                Where-Object {$_.Level -le 2 -and $_.BewertungWert -ne 'gruen' -and $AdminAusnahmen -contains $_.Benutzer} | 
                    Sort-Object -Property Level,Benutzer
            $AnzahlAdminsBad = ($Tab | measure-object).count
            
            if ($Tab) {
                $HTML += erstelle-HTMLReport `
                    -Daten $Tab `
                    -Headline "$AnzahlAdminsBad/$AnzahlAdmins privilegierte Accounts sind gefaehrdet, aber manuell ausgenommen" `
                    -Spalten ('Benutzer','Level','Policy',,'Lockout','IsProtectedUser:ProtectedUser','Enabled','Description','SchedTasks','ServiceUser','LastLogonDate:LastLogon',
                              'CannotChangePassword:PWfixed','PasswordNeverExpires:PWendless','PasswordLastSet:PWlastset','PasswordExpireDate:PWexpire',
                              'PWcomplex','PWreversible','PWlength','PWhistory','PWminAge','KRBpreAuth','SCLogonRequired:requireSC','Bewertung') `
                    -Muster blau
            }

        # sichere Admins
            $Tab = $Benutzer | 
                Where-Object {$_.Level -le 2 -and $_.BewertungWert -eq 'gruen'} |  
                    Sort-Object -Property Level,Benutzer            
            $AnzahlAdminsOK  = ($Tab | measure-object).count

            if ($Tab) {
                $HTML += erstelle-HTMLReport `
                    -Daten $Tab `
                    -Headline "$AnzahlAdminsOK/$AnzahlAdmins privilegierte Accounts sind sicher" `
                    -Spalten ('Benutzer','Level','Policy',,'Lockout','IsProtectedUser:ProtectedUser','Enabled','Description','SchedTasks','ServiceUser','LastLogonDate:LastLogon',
                              'CannotChangePassword:PWfixed','PasswordNeverExpires:PWendless','PasswordLastSet:PWlastset','PasswordExpireDate:PWexpire',
                              'PWcomplex','PWreversible','PWlength','PWhistory','PWminAge','KRBpreAuth','SCLogonRequired:requireSC','Bewertung') `
                    -Farbdefinition $FarbFilter `
                    -Muster gruen
            }
        
        # geplante AdminAufgaben
            erstelle-FarbFilter -Spalte Level -gelb '$_.Level -eq 2' -rot '$_.Level -eq 1' -neuerFilter
            
            $Tab1 = $SchedTasks | Where-Object {$Admins -contains $_.userid -and $AdminAusnahmen -notcontains $_.userid} |
                Select-Object -Property  `
                    @{n='Benutzer';e={$_.UserID}},
                    @{n='Level';e={
                        $User = $_.UserID
                        ($Benutzer | Where-Object {$_.Benutzer -eq $User}).Level
                    }},
                    @{n='Hostname';e={$_.ComputerName}},
                    @{n='TaskName';e={$_.name}},
                    @{n='TaskAction';e={$_.Action}},
                    @{n='LastRunTime';e={$_.LastRunTime}},
                    @{n='NextRunTime';e={if ($_.NextRunTime -eq (get-date '30.12.1899')) { '---' } else {$_.NextRunTime }}},
                    @{n='Description';e={$_.Description}}
            if ($Tab1) {
                $HTML += erstelle-HTMLReport `
                    -Daten $Tab1 `
                    -Headline "$($Tab1.count) geplante Aufgaben, die mit AdminKennung laufen" `
                    -Spalten ('Benutzer','Level','Hostname','TaskName','TaskAction','LastRunTime','NextRunTime','Description') `
                    -Muster rot `
                    -Farbdefinition $FarbFilter
            }

        # ServiceAdmins
            $Tab2 = $Services | 
                Where-Object { $Admins -contains $_.Benutzer -and $AdminAusnahmen -notcontains $_.Benutzer} |                 
                    Select-Object -Property Benutzer,Server,Service,State,StartMode,Pfad, `
                        @{n='Level';e={
                            $SAM = $_.Benutzer 
                            ($Benutzer | Where-Object {$_.Benutzer -eq $SAM}).Level
                        }} | sort Level,Benutzer,Server
            if ($Tab2) {
                $HTML += erstelle-HTMLReport `
                    -Daten $Tab2 `
                    -Headline "$($Tab2.count) Services, die mit AdminKennung laufen" `
                    -Spalten ('Benutzer','Level','Server','Service','State','StartMode','Pfad') `
                    -Muster rot `
                    -Farbdefinition $FarbFilter
            }

        # Statistik
            $ScriptDauer = [int] ((get-date) - $ScriptStart).totalseconds
            $Statistik = "
            <p></p>
            <h3>Zusammenfassung:</h3>

            <table>
                <tr><td>  Generiert auf:             </td><td> $($env:computername)                                                </td></tr>
                <tr><td>  Scriptversion:             </td><td> $($ScriptVersion)                                                   </td></tr>
                <tr><td>  Scan-Dauer                 </td><td> $($ScriptDauer) sec                                                 </td></tr>        
                <tr><td>  Anzahl Server:             </td><td> $($Memberserver.count)                                              </td></tr>  
                <tr><td>  ...davon erreichbar:       </td><td> $($Memberserver.count - ($Error | ForEach-Object {$_.TargetObject} | Group-Object).count) </td></tr>             
                <tr><td>  ...mit Fehler:             </td><td> $(($Error | ForEach-Object {$_.TargetObject} | Group-Object).count) </td></tr>             
                <tr><td>  Anzahl Gruppen:            </td><td> $($AdminGroups.count)                                               </td></tr>
                <tr><td>  Anzahl Admins:             </td><td> $(($Benutzer | Where-Object {$_.Level -le 2}).count)                </td></tr>
                <tr><td>  ...mit Level 1:            </td><td> $(($Benutzer | Where-Object {$_.Level -eq 1}).count)                </td></tr>
                <tr><td>  ...mit Level 2:            </td><td> $(($Benutzer | Where-Object {$_.Level -eq 2}).count)                </td></tr>
                <tr><td>  ...gefaehrdet:             </td><td> $(($Benutzer | Where-Object {$_.Level -le 2 -and $_.Bewertung -ne ''}).count)           </td></tr>
                <tr><td>  ...sicher:                 </td><td> $(($Benutzer | Where-Object {$_.Level -le 2 -and $_.Bewertung -eq ''}).count)           </td></tr>
                <tr><td>  Anzahl Admins mit Task:    </td><td> $($Tab1.count)                                                      </td></tr>
                <tr><td>  Anzahl Admins mit Service: </td><td> $($Tab2.count)                                                      </td></tr>
            </table>
            "
            $HTML += $Statistik
         
        # Abschluss               
            $HTML += '</body>'
            $HTML += '</html>'

            $Datum = ((Get-Date -Format u) -split ' ')[0]

            $HTML | Out-File "$ScriptDir\Logs\$Datum-PrivilegedADUser-Analyse.htm"
        
            $global:HTMLMail = $HTML -join "`r`n"
    }

    function schreibe-Bewertung {
        Write-Host "bewerte Ergebnis" -ForegroundColor White
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name PWExpireInDays -Value ""
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name Bewertung -Value ''
        $global:Benutzer | Add-Member -MemberType NoteProperty -Name BewertungWert -Value ''

        $Ausnahmen = Get-Variable -name "RULE_*" | Select-Object -Property `
                        @{n='Benutzer' ; e={$_.name -replace 'RULE_'}},
                        @{n='Ausnahme' ; e={$_.Value}}

        $global:Benutzer | ForEach-Object {
            $Bewertung = @()
            $BewertungWert = 'gruen'
                        
            if ($_.Benutzer -eq 'KRBTGT') {
                $_.PasswordExpireDate = (Get-Date -date ([datetime] $_.PasswordLastSet).addDays($KRBTGT_PWDLaufzeit) -Format u) -replace 'Z'
                $_.LastLogonDate = "always"
                $notKRBTGT = $false
            } else {
                $notKRBTGT = $true
            }
            
            if ($_.PasswordExpireDate -eq $null) {
                $PWDTage = 999
                $_.PasswordExpireDate = 'never'
            } else {
                $PWDTage = [int] ((get-date ($_.PasswordExpireDate)) - (get-date)).totaldays
                $_.PWExpireInDays = $PWDTage
            }   

            if ($PWDTage -gt $PWDAblauf_Achtung -and $PWDTage -lt $PWDAblauf_Warnung)  {$Bewertung += '-PWD laeuft ab'         ; $BewertungWert = 'gelb' }
            if ($PWDTage -gt 0 -and $PWDTage -le $PWDAblauf_Achtung)                   {$Bewertung += '-PWD laeuft bald ab'    ; $BewertungWert = 'rot'  }
            if ($PWDTage -le 0)                                                        {$Bewertung += '-PWD abgelaufen'        ; $BewertungWert = 'rot'  }
            if ($_.PWlength -le $PWmindestLaenge -and -not $_.SCLogonRequired)         {$Bewertung += '-kurzes PW'             ; $BewertungWert = 'rot'  }
            
            if ($notKRBTGT) {
                if ($_.LastLogonDate -eq 'never')                                      {$Bewertung += '-nicht verwendet'       ; $BewertungWert = 'gelb' }
                
                if ($_.PasswordNeverExpires -or $_.CannotChangePassword)               {$Bewertung += '-statisches PWD'        ; $BewertungWert = 'rot'  }
                if ($_.IsProtectedUser -eq $false -and $_.Level -eq 1)                 {$Bewertung += '-ProtectedUser'         ; $BewertungWert = 'rot'  }                        
                if ($_.IsProtectedUser -eq $true)                                      {$Bewertung += '+ProtectedUser'                                   }                        
                if ($_.PasswordNotRequired)                                            {$Bewertung += '-kein PWD erforderlich' ; $BewertungWert = 'rot'  }            
                if ($_.SchedTasks)                                                     {$Bewertung += '-Taskuser'              ; $BewertungWert = 'rot'  }
                if ($_.ServiceUser)                                                    {$Bewertung += '-ServiceUser'           ; $BewertungWert = 'rot'  }
                if ($_.Lockout -eq 0)                                                  {$Bewertung += '-kein Lockout'          ; $BewertungWert = 'rot'  }
                if ($_.PWcomplex -eq $false -and $_.PWlength -le $PWsichereLaenge)     {$Bewertung += '-schwaches PW'          ; $BewertungWert = 'rot'  }                
                if ($_.PWreversible)                                                   {$Bewertung += '-reversibles PW'        ; $BewertungWert = 'rot'  }
                if ($_.KRBpreAuth -eq $false)                                          {$Bewertung += '-kein PreAuth'          ; $BewertungWert = 'rot'  }
                if ($_.PWHistory -le $PWhistory_Warnung -or $_.PWMinAge -eq 0)         {$Bewertung += '-PWHistory'             ; $BewertungWert = 'rot'  }                
                if ($_.SCLogonRequired)                                                {$Bewertung += '+SmardCard-protected'                             }
                            
                if ($_.SCLogonRequired -and $POS_SCRequire -eq 'j')                    {                                         $BewertungWert = 'gruen'}            
                if ($_.IsProtectedUser -eq $true -and $POS_ProtectedUser -eq 'j')      {                                         $BewertungWert = 'gruen'}            
            }
 
            $_.Bewertung     = ( $Bewertung | Sort-Object ) -join ', '
            $_.BewertungWert = $BewertungWert
        }     
    }

    function MAIN { 
        cls   
        $ErrorActionPreference = "stop"

        # starte Runtime-Logfile
            Start-Transcript -Path "$ScriptDir\RunTime.log"

        # erstelle Variablen            
            Get-Variable -Scope global | Where-Object {$globaleVariablen -contains $_.name} | Remove-Variable -Scope global
            $globaleVariablen | ForEach-Object { New-Variable -Name $_ -Value @() -Scope global }
            $Global:ScriptStart = (get-date)

            lese-Variablen -Datei "$ScriptDir\PrivilegedADUser-Analyse.ini" -VarPrefix ''
            
        # lade PW-Infos
            $DomainDN = "DC=$(($env:USERDNSDOMAIN -split '\.') -join ",DC=")"            
            [adsi]$Domain = "LDAP://$DomainDN"
            
            $global:PP = New-Object -TypeName PSObject
            $global:PP | Add-Member -MemberType NoteProperty -Name ADminPwdAge_Day       -Value ([math]::Abs((New-TimeSpan -Seconds ($Domain.ConvertLargeIntegerToInt64($Domain.minPwdAge.Value) / 10000000)).Days))
            $global:PP | Add-Member -MemberType NoteProperty -Name ADmaxPwdAge_Day       -Value ([math]::Abs((New-TimeSpan -Seconds ($Domain.ConvertLargeIntegerToInt64($Domain.maxPwdAge.Value) / 10000000)).Days))
            $global:PP | Add-Member -MemberType NoteProperty -Name ADlockoutDuration_Min -Value ([math]::Abs((New-TimeSpan -Seconds ($Domain.ConvertLargeIntegerToInt64($Domain.lockoutDuration.Value) / 10000000)).Minutes))
            $global:PP | Add-Member -MemberType NoteProperty -Name ADlockoutObserv_Min   -Value ([math]::Abs((New-TimeSpan -Seconds ($Domain.ConvertLargeIntegerToInt64($Domain.lockOutObservationWindow.Value) / 10000000)).Minutes))
            $global:PP | Add-Member -MemberType NoteProperty -Name ADpwdHistoryLength    -Value $Domain.pwdHistoryLength.Value
            $global:PP | Add-Member -MemberType NoteProperty -Name ADminPwdLength        -Value $Domain.minPwdLength.Value
            $global:PP | Add-Member -MemberType NoteProperty -Name ADlockoutThreshold    -Value $Domain.lockoutThreshold.Value
            $global:PP | Add-Member -MemberType NoteProperty -Name PWcomplex             -Value ([bool] ([convert]::ToString($Domain.pwdProperties.Value,2) -band 00001))
            $global:PP | Add-Member -MemberType NoteProperty -Name PWreversible          -Value ([bool] ([convert]::ToString($Domain.pwdProperties.Value,2) -band 10000))
          
        # starte Funktionen
            finde-MemberServer    
            finde-AdminGruppen
            finde-Administratoren
            finde-ScheduledTasks
            finde-AdminsMitScheduledTasks
            finde-Services
            finde-AdminsMitServices
            schreibe-Bewertung
            erstelle-HTML

        # sende Mail
            if ($MailAktiv -eq 'ja') {
                Write-Host "sende InfoMail" -ForegroundColor White
                $usr  = "anonymous"
                $pwd  = ConvertTo-SecureString -String "anonymous" -AsPlainText -Force
                $cred = New-Object System.Management.Automation.PSCredential($usr,$pwd)

                Send-MailMessage `
                    -to         $MailAn `
                    -from       $MailVon `
                    -Subject    "Privileged-ADUser-Analyse" `
                    -body       $HTMLMail `
                    -BodyAsHtml `
                    -SmtpServer $MailServer `
                    -Credential $cred `
                    -Encoding default
            } else {
                Write-Host "zeige Report" -ForegroundColor White
                Get-Process | Where-Object { $_.name -eq 'iexplore' } | Stop-Process
                $Datum = ((Get-Date -Format u) -split ' ')[0]
                Invoke-Expression -Command "$ScriptDir\Logs\$Datum-PrivilegedADUser-Analyse.htm"
            }

        # stoppe Runtime-Logfile
            Stop-Transcript | Out-Null
    }

# Start
    MAIN
    